package com.sun.tools.javac.comp;

import java.util.Arrays;

import com.sun.tools.javac.code.Attribute;
import com.sun.tools.javac.tree.JCTree;

/**
 * Represents the place in a compiled class file where an annotation points. All
 * annotations have them, but they are only meaningful for extended annotations.
 */
public final class AnnotationTarget {

    /*
     * ref is (unfortunately) required, because at codegen time, generics have
     * been translated out of the tree.
     * 
     * You can't visit the locals and get the annotations in their types that
     * way (for instance), but you do have all the attributes already entered on
     * the method.
     */
    public JCTree ref; // should always be set; for non-generics, at parse
                        // time, for generics, at resolve time

    public Attribute.Compound attr;
    
    public TargetType type;

    public int offset;

    public int start_pc;

    public int length;

    public int index;

    public int[] location;

    public int parameter; // For method and type parameters.

    public int bound;

    // doesn't override equals on purpose

    public AnnotationTarget() {
        ref = null;
        attr = null;
        type = TargetType.UNKNOWN;
        offset = -1;
        start_pc = -1;
        length = -1;
        index = -1;
        location = new int[0];
        parameter = -1;
        bound = -1;
    }

    public String toString() {
        String locString = Arrays.toString(location);
        return String.format("%s\n  %3d / %3d %3d %3d / %2d %2d / %-16s", type, 
        	offset, start_pc, length, index, parameter, bound, locString);
    }
}
