/*
 * Copyright 2003 Sun Microsystems, Inc.  All Rights Reserved.
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
 *
 * This code is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License version 2 only, as
 * published by the Free Software Foundation.  Sun designates this
 * particular file as subject to the "Classpath" exception as provided
 * by Sun in the LICENSE file that accompanied this code.
 *
 * This code is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
 * version 2 for more details (a copy is included in the LICENSE file that
 * accompanied this code).
 *
 * You should have received a copy of the GNU General Public License version
 * 2 along with this work; if not, write to the Free Software Foundation,
 * Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 * Please contact Sun Microsystems, Inc., 4150 Network Circle, Santa Clara,
 * CA 95054 USA or visit www.sun.com if you need additional information or
 * have any questions.
 */

package sun.print;

import javax.print.attribute.EnumSyntax;
import javax.print.attribute.PrintRequestAttribute;

/**
 * Class DialogTypeSelection is a printing attribute class, an enumeration, 
 * that indicates the dialog to be used for PrintService selection.
 * If NATIVE is specified, the native print dialog is displayed.
 * If COMMON is specified, a cross-platform print dialog is displayed.
 *
 * <P>
 * <B>IPP Compatibility:</B> This is not an IPP attribute.
 * <P>
 *
 */
public final class DialogTypeSelection extends EnumSyntax 
	implements PrintRequestAttribute {

    /**
     * 
     */
    public static final DialogTypeSelection 
	NATIVE = new DialogTypeSelection(0);	
	    
    /**
     * 
     */
    public static final DialogTypeSelection 
	COMMON = new DialogTypeSelection(1);
    
    /**
     * Construct a new dialog type selection enumeration value with the 
     * given integer value. 
     *
     * @param  value  Integer value.
     */
    protected DialogTypeSelection(int value) {
		super(value);
    }

    private static final String[] myStringTable = {
	"native", "common"};


    private static final DialogTypeSelection[] myEnumValueTable = {
	NATIVE,
	COMMON
    };

    /**
     * Returns the string table for class DialogTypeSelection.
     */
    protected String[] getStringTable() {
	return myStringTable;
    }

    /**
     * Returns the enumeration value table for class DialogTypeSelection.
     */
    protected EnumSyntax[] getEnumValueTable() {
	return myEnumValueTable;
    }


   /**
     * Get the printing attribute class which is to be used as the "category" 
     * for this printing attribute value.
     * <P>
     * For class DialogTypeSelection the category is class 
     * DialogTypeSelection itself. 
     *
     * @return  Printing attribute class (category), an instance of class
     *          {@link java.lang.Class java.lang.Class}.
     */
    public final Class getCategory() {
        return DialogTypeSelection.class;
    }
   

    /**
     * Get the name of the category of which this attribute value is an 
     * instance. 
     * <P>
     * For class DialogTypeSelection the category name is
     * <CODE>"dialog-type-selection"</CODE>. 
     *
     * @return  Attribute category name.
     */
    public final String getName() {
        return "dialog-type-selection";
    }

}
