/*
 * Copyright 2006 Sun Microsystems, Inc.  All Rights Reserved.
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
 *
 * This code is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License version 2 only, as
 * published by the Free Software Foundation.
 *
 * This code is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
 * version 2 for more details (a copy is included in the LICENSE file that
 * accompanied this code).
 *
 * You should have received a copy of the GNU General Public License version
 * 2 along with this work; if not, write to the Free Software Foundation,
 * Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 * Please contact Sun Microsystems, Inc., 4150 Network Circle, Santa Clara,
 * CA 95054 USA or visit www.sun.com if you need additional information or
 * have any questions.
 */

/*
 * @test @(#)ChunkedErrorStream.java	1.5 07/05/05
 * @bug 6488669
 * @run main/othervm ChunkedErrorStream
 * @summary Bad interaction between ChunkedInputStream and HttpURLConnection$ErrorStream.getErrorStream
 */

import java.net.*;
import java.io.*;
import com.sun.net.httpserver.*;

/**
 * 1) Http server that responds with an error code (>=400) 
 *    and a chunked response body. It also indicates that 
 *    the connection will be closed.
 * 2) Client sends request to server and tries to 
 *    getErrorStream()
 */

public class ChunkedErrorStream
{
    com.sun.net.httpserver.HttpServer httpServer;

    static {
	// Enable ErrorStream buffering
	System.getProperties().setProperty("sun.net.http.errorstream.enableBuffering", "true");
    }

    public static void main(String[] args) {
	new ChunkedErrorStream();
    }

    public ChunkedErrorStream() {
	try {
            startHttpServer();
            doClient();
        } catch (IOException ioe) {
            ioe.printStackTrace();
        }  finally {
            httpServer.stop(1);
        }

    }

    void doClient() {
	HttpURLConnection uc = null;
        try {
	    InetSocketAddress address = httpServer.getAddress();

            URL url = new URL("http://localhost:" + address.getPort() + "/test/badRequest");
            uc = (HttpURLConnection)url.openConnection();
            uc.getInputStream();

	    throw new RuntimeException("Failed: getInputStream should throw and IOException");
	}  catch (IOException e) {
	    // This is what we expect to happen.
	    InputStream es = uc.getErrorStream();
	    System.out.println(es);
	    byte[] ba = new byte[1024];
	    int count = 0, ret;
	    try {
		while ((ret = es.read(ba)) != -1)
		    count += ret;
	    	es.close();
	    } catch  (IOException ioe) {
		ioe.printStackTrace();
	    }

	    if (count == 0)
	 	throw new RuntimeException("Failed: ErrorStream returning 0 bytes");

	    System.out.println("count = " + count);

        }
    }

    /**
     * Http Server
     */
    void startHttpServer() throws IOException {
        httpServer = com.sun.net.httpserver.HttpServer.create(new InetSocketAddress(0), 0);

        // create HttpServer context
        HttpContext ctx = httpServer.createContext("/test/", new MyHandler());

        httpServer.start();
    }

    class MyHandler implements HttpHandler {
        public void handle(HttpExchange t) throws IOException {
	    InputStream is = t.getRequestBody();
	    byte[] ba = new byte[1024];
	    while (is.read(ba) != -1);
	    is.close();

	    Headers resHeaders = t.getResponseHeaders();
	    resHeaders.add("Connection", "close");
            t.sendResponseHeaders(404, 0);
	    OutputStream os = t.getResponseBody();

	    // actual data doesn't matter. Just send 2K worth.
	    byte b = 'a';
	    for (int i=0; i<2048; i++)
		os.write(b);

	    os.close();
            t.close();
        }
    }
}
